package com.flozano.mim.air.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.object.SqlUpdate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.stereotype.Repository;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.air.AirlineCode;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.BookingClass;
import com.flozano.mim.air.Segment;
import com.flozano.mim.air.SegmentNotFoundException;
import com.flozano.mim.air.dao.SegmentDAO;

@Repository
public class JDBCSegmentDAOImpl implements SegmentDAO {

	private static final String SQL_INSERT = "INSERT INTO segment (origin, destination, departure, airline, flight_number, booking_class, loaded_member_number, member_account_statement) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
	private static final int[] SQL_INSERT_TYPES = new int[] { Types.CHAR,
			Types.CHAR, Types.TIMESTAMP, Types.CHAR, Types.INTEGER, Types.CHAR,
			Types.VARCHAR, Types.BIGINT };
	private static final String[] SQL_INSERT_GENERATED_KEYS = new String[] { "id" };

	private static final String PARTIAL_QUERY_WITH_JOINS = "SELECT s.*, a.name AS airline_name, CONCAT(b.cabin,' (', b.id,')') AS booking_class_name, ap1.name AS destination_name, ap2.name AS origin_name, st.miles AS miles FROM member m, member_account_statement st, segment s, airline a, booking_class b, airport ap1, airport ap2 WHERE s.airline=a.id AND s.booking_class=b.id AND s.destination=ap1.id AND s.origin=ap2.id AND m.id=st.member AND st.id=s.member_account_statement";
	private final JdbcTemplate jdbcTemplate;
	private final SqlUpdate insertOperation;

	@Autowired
	public JDBCSegmentDAOImpl(DataSource dataSource) {
		jdbcTemplate = new JdbcTemplate(dataSource);
		insertOperation = new SqlUpdate(dataSource, SQL_INSERT,
				SQL_INSERT_TYPES, 1);
		insertOperation.setGeneratedKeysColumnNames(SQL_INSERT_GENERATED_KEYS);
	}

	@Override
	public long insert(Segment segment, Long statementId) {
		GeneratedKeyHolder k = new GeneratedKeyHolder();
		int n = insertOperation.update(new Object[] {
				segment.getOrigin().toString(),
				segment.getDestination().toString(), segment.getDeparture(),
				segment.getAirline().toString(), segment.getFlightNumber(),
				segment.getBookingClass().toString(),
				segment.getLoadedMemberNumber().toString(), statementId }, k);
		if (n != 1)
			throw new IncorrectResultSizeDataAccessException(1);
		return (Long) k.getKey();
	}

	@Override
	public List<Segment> find(MemberNumber memberNumber, Date from, Date until) {
		return jdbcTemplate
				.query(PARTIAL_QUERY_WITH_JOINS
						+ " AND m.member_number=? AND s.departure BETWEEN ? AND ? ORDER BY departure DESC",
						ROW_MAPPER_COMPLETE_FOR_REPORT,
						memberNumber.toString(), from, until);
	}

	private static final RowMapper<Segment> ROW_MAPPER_COMPLETE_FOR_REPORT = new RowMapper<Segment>() {

		@Override
		public Segment mapRow(ResultSet rs, int rowNum) throws SQLException {
			Segment s = new Segment();
			s.setAirline(new AirlineCode(rs.getString("airline")));
			s.setAirlineName(rs.getString("airline_name"));
			s.setBookingClass(new BookingClass(rs.getString("booking_class")));
			s.setBookingClassName(rs.getString("booking_class_name"));
			s.setDeparture(rs.getDate("departure"));
			s.setDestination(new AirportCode(rs.getString("destination")));
			s.setDestinationName(rs.getString("destination_name"));
			s.setFlightNumber(rs.getInt("flight_number"));
			s.setId(rs.getLong("id"));
			s.setOrigin(new AirportCode(rs.getString("origin")));
			s.setOriginName(rs.getString("origin_name"));
			s.setMiles(rs.getInt("miles"));
			return s;
		}
	};

	@Override
	public List<Segment> findOrphaned() {
		// TODO Implement find orphaned
		return new LinkedList<Segment>();
	}

	@Override
	public boolean exists(Segment segment) {
		return jdbcTemplate
				.queryForInt(
						"SELECT COUNT(id) FROM segment WHERE origin=? AND destination=? AND departure=? AND airline=? AND flight_number=? AND loaded_member_number=?",
						segment.getOrigin().toString(), segment
								.getDestination().toString(), segment
								.getDeparture(), segment.getAirline()
								.toString(), segment.getFlightNumber(), segment
								.getLoadedMemberNumber().toString()) > 0;
	}

	@Override
	public void update(long memberId, long segmentId, AirlineCode airline, int flightNumber,
			AirportCode origin, AirportCode destination, Date departure) {
		int n = jdbcTemplate
				.update("UPDATE segment SET origin=?, destination=?, airline=?, flight_number=?, departure=? WHERE id=? AND member_account_statement IN (select id FROM member_account_statement WHERE member=?)",
						origin.toString(), destination.toString(),
						airline.toString(), flightNumber, departure, segmentId, memberId);
		if (n != 1) {
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public Segment get(long memberId, long segmentId) {
		List<Segment> segments = jdbcTemplate.query(PARTIAL_QUERY_WITH_JOINS
				+ " AND st.member=? AND s.id=?",
				ROW_MAPPER_COMPLETE_FOR_REPORT, memberId, segmentId);
		switch (segments.size()) {
		case 0:
			throw new SegmentNotFoundException(segmentId);
		case 1:
			return segments.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}
}
