package com.flozano.mim.member.dao.jdbc;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Date;
import java.util.List;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.object.SqlUpdate;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.stereotype.Repository;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.member.Called;
import com.flozano.mim.member.EmailAddress;
import com.flozano.mim.member.EmailAddressAlreadyExistsException;
import com.flozano.mim.member.FullName;
import com.flozano.mim.member.Member;
import com.flozano.mim.member.MemberNotFoundException;
import com.flozano.mim.member.Password;
import com.flozano.mim.member.PhoneNumber;
import com.flozano.mim.member.PhoneNumberAlreadyExistsException;
import com.flozano.mim.member.PostalAddress;
import com.flozano.mim.member.dao.MemberDAO;
import com.flozano.mim.util.SortDirection;

@Repository
public class JDBCMemberDAOImpl implements MemberDAO {

	private static final String SQL_INSERT = "INSERT INTO member (member_number, password, created_at, modified_at, version, called, first_name, last_names, email_address, phone_number, address, city, state_or_region, zip_code, country) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)";
	private static final int[] SQL_INSERT_TYPES = new int[] { Types.CHAR,
			Types.VARCHAR, Types.TIMESTAMP, Types.TIMESTAMP, Types.INTEGER,
			Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR,
			Types.VARCHAR, Types.VARCHAR, Types.VARCHAR, Types.VARCHAR,
			Types.VARCHAR, Types.CHAR };
	private final JdbcTemplate jdbcTemplate;

	private final SqlUpdate insertOperation;

	@Autowired
	public JDBCMemberDAOImpl(DataSource dataSource) {
		this.jdbcTemplate = new JdbcTemplate(dataSource);
		insertOperation = new SqlUpdate(dataSource, SQL_INSERT,
				SQL_INSERT_TYPES, 1);
		insertOperation.setGeneratedKeysColumnNames(new String[] { "id" });
	}

	@Override
	public long getIdByMemberNumber(MemberNumber memberNumber)
			throws MemberNotFoundException {

		List<Long> list = jdbcTemplate.queryForList(
				"SELECT id FROM member WHERE member_number = ?", Long.class,
				memberNumber.toString());
		switch (list.size()) {
		case 0:
			throw new MemberNotFoundException(memberNumber);
		case 1:
			return list.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}

	}

	@Override
	public MemberNumber getMemberNumberByID(long memberId) {
		List<String> list = jdbcTemplate.queryForList(
				"SELECT member_number FROM member WHERE id = ?", String.class,
				memberId);
		switch (list.size()) {
		case 0:
			throw new MemberNotFoundException(memberId);
		case 1:
			return new MemberNumber(list.get(0));
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	protected boolean check(String field, String value,
			MemberNumber expectedOwner) {
		List<String> result = jdbcTemplate.queryForList(
				"SELECT member_number FROM member WHERE " + field + " = ?",
				String.class, value);
		String actualOwner = result.size() > 0 ? result.get(0) : null;
		if (expectedOwner == null) {
			if (actualOwner != null) {
				return false;
			}
		} else {
			if (!expectedOwner.toString().equals(actualOwner)) {
				return false;
			}
		}
		return true;
	}

	protected void check(EmailAddress emailAddress, PhoneNumber phoneNumber,
			MemberNumber expectedOwner) {
		if (!check("email_address", emailAddress.toString(), expectedOwner)) {
			throw new EmailAddressAlreadyExistsException(emailAddress);
		}
		if (!check("phone_number", phoneNumber.toString(), expectedOwner)) {
			throw new PhoneNumberAlreadyExistsException(phoneNumber);
		}
	}

	@Override
	public long insert(MemberNumber memberNumber, FullName fullName,
			Password password, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress postalAddress)
			throws EmailAddressAlreadyExistsException,
			PhoneNumberAlreadyExistsException {
		check(emailAddress, phoneNumber, null);
		Date now = new Date();
		KeyHolder k = new GeneratedKeyHolder();
		insertOperation.update(
				new Object[] { memberNumber.toString(), password.toString(),
						now, now, 0, fullName.getCalled().toString(),
						fullName.getFirstName(), fullName.getLastName(),
						emailAddress.toString(), phoneNumber.toString(),
						postalAddress.getAddress(), postalAddress.getCity(),
						postalAddress.getStateOrRegion(),
						postalAddress.getZipCode(),
						postalAddress.getCountry().toString() }, k);
		return (Long) k.getKey();
	}

	@Override
	public void update(MemberNumber memberNumber, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress address) {

		check(emailAddress, phoneNumber, memberNumber);

		int n = jdbcTemplate
				.update("UPDATE member SET email_address=?, phone_number=?, address=?, city=?, state_or_region=?, zip_code=?, country=? WHERE member_number = ?",
						emailAddress.toString(), phoneNumber.toString(),
						address.getAddress(), address.getCity(),
						address.getStateOrRegion(), address.getZipCode(),
						address.getCountry(), memberNumber.toString());
		switch (n) {
		case 0:
			throw new MemberNotFoundException(memberNumber);
		case 1:
			return;
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public void update(long memberId, FullName fullName, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress address) {
		// TODO refactor this
		String memberNumber = jdbcTemplate.queryForObject(
				"SELECT member_number FROM member WHERE id = ?", String.class,
				memberId);
		check(emailAddress, phoneNumber, new MemberNumber(memberNumber));

		int n = jdbcTemplate
				.update("UPDATE member SET email_address=?, phone_number=?, address=?, city=?, state_or_region=?, zip_code=?, country=? WHERE id = ?",
						emailAddress.toString(), phoneNumber.toString(),
						address.getAddress(), address.getCity(),
						address.getStateOrRegion(), address.getZipCode(),
						address.getCountry(), memberId);
		switch (n) {
		case 0:
			throw new MemberNotFoundException(memberId);
		case 1:
			return;
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public int updatePassword(MemberNumber memberNumber, Password oldPassword,
			Password newPassword) {
		return jdbcTemplate
				.update("UPDATE member SET password = ? WHERE member_number=? AND password=?",
						newPassword.toString(), memberNumber.toString(),
						oldPassword.toString());
	}

	@Override
	public Member get(MemberNumber memberNumber) throws MemberNotFoundException {
		List<Member> result = jdbcTemplate.query(
				"SELECT * FROM member WHERE member_number = ?", ROW_MAPPER,
				memberNumber.toString());
		switch (result.size()) {
		case 0:
			throw new MemberNotFoundException(memberNumber);
		case 1:
			return result.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public Member get(long memberId) throws MemberNotFoundException {
		List<Member> result = jdbcTemplate.query(
				"SELECT * FROM member WHERE id = ?", ROW_MAPPER, memberId);
		switch (result.size()) {
		case 0:
			throw new MemberNotFoundException(memberId);
		case 1:
			return result.get(0);
		default:
			throw new IncorrectResultSizeDataAccessException(1);
		}
	}

	@Override
	public List<Member> find(String search, String sortProperty,
			SortDirection direction, int limit, int offset) {
		return jdbcTemplate
				.query("SELECT * FROM member WHERE (member_number LIKE ?) OR (first_name LIKE ?) OR (last_names LIKE ?) OR (email_address LIKE ?) OR (phone_number LIKE ?) OR (address LIKE ?) OR (city LIKE ?) OR (state_or_region LIKE ?) OR (country LIKE ?) LIMIT ? OFFSET ?",
						ROW_MAPPER, search, search, search, search, search,
						search, search, search, search, limit, offset);
	}

	@Override
	public int count(String search) {
		return jdbcTemplate
				.queryForInt(
						"SELECT COUNT(*) FROM member WHERE (member_number LIKE ?) OR (first_name LIKE ?) OR (last_names LIKE ?) OR (email_address LIKE ?) OR (phone_number LIKE ?) OR (address LIKE ?) OR (city LIKE ?) OR (state_or_region LIKE ?) OR (country LIKE ?)",
						search, search, search, search, search, search, search,
						search, search);
	}

	private static final RowMapper<Member> ROW_MAPPER = new RowMapper<Member>() {

		@Override
		public Member mapRow(ResultSet rs, int rowNum) throws SQLException {
			Member m = new Member();
			m.setId(rs.getLong("id"));
			m.setMemberNumber(new MemberNumber(rs.getString("member_number")));
			m.setEmailAddress(new EmailAddress(rs.getString("email_address")));
			m.setPhoneNumber(new PhoneNumber(rs.getString("phone_number")));
			m.getFullName().setCalled(Called.valueOf(rs.getString("called")));
			m.getFullName().setFirstName(rs.getString("first_name"));
			m.getFullName().setLastName(rs.getString("last_names"));
			m.getPostalAddress().setAddress(rs.getString("address"));
			m.getPostalAddress().setCity(rs.getString("city"));
			m.getPostalAddress().setCountry(rs.getString("country"));
			m.getPostalAddress().setStateOrRegion(
					rs.getString("state_or_region"));
			m.getPostalAddress().setZipCode(rs.getString("zip_code"));
			m.setEmailAddress(new EmailAddress(rs.getString("email_address")));
			return m;
		}
	};

}
