package com.flozano.mim.member.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.air.Accrual;
import com.flozano.mim.air.AccrualService;
import com.flozano.mim.air.AirlineCode;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.EliteBonusService;
import com.flozano.mim.air.EliteStatus;
import com.flozano.mim.air.Segment;
import com.flozano.mim.air.dao.SegmentDAO;
import com.flozano.mim.member.AccountReport;
import com.flozano.mim.member.AccountService;
import com.flozano.mim.member.FlightsReport;
import com.flozano.mim.member.MemberNotFoundException;
import com.flozano.mim.member.Note;
import com.flozano.mim.member.dao.MemberAccountStatementDAO;
import com.flozano.mim.member.dao.MemberDAO;
import com.flozano.mim.member.dao.NoteDAO;
import com.flozano.mim.util.Check;

@Service("accountService")
@Transactional
public class AccountServiceImpl implements AccountService {

	private final MemberDAO memberDAO;
	private final MemberAccountStatementDAO memberAccountStatementDAO;
	private final SegmentDAO segmentDAO;
	private final NoteDAO noteDAO;

	private final AccrualService accrualService;
	private final EliteBonusService eliteBonusService;

	@Autowired
	public AccountServiceImpl(AccrualService accrualService,
			EliteBonusService eliteBonusService, MemberDAO memberDAO,
			SegmentDAO segmentDAO,
			MemberAccountStatementDAO memberAccountStatementDAO, NoteDAO noteDAO) {
		Check.notNull(accrualService, eliteBonusService, memberDAO, segmentDAO,
				memberAccountStatementDAO, noteDAO);
		this.segmentDAO = segmentDAO;
		this.memberDAO = memberDAO;
		this.eliteBonusService = eliteBonusService;
		this.accrualService = accrualService;
		this.memberAccountStatementDAO = memberAccountStatementDAO;
		this.noteDAO = noteDAO;
	}

	@Override
	public AccountReport getAccountReport(MemberNumber memberNumber) {
		int totalMiles = memberAccountStatementDAO.totalMiles(memberNumber);

		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		Date when = cal.getTime();

		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.add(Calendar.YEAR, -1);
		Date from = cal.getTime();

		int pastYearMiles = memberAccountStatementDAO.sumMilesAccruedBetween(
				memberNumber, from, when);

		cal.setTime(when);
		cal.add(Calendar.MONTH, 1);
		when = cal.getTime();

		cal.setTime(from);
		cal.add(Calendar.MONTH, 1);
		from = cal.getTime();

		int nextMiles = memberAccountStatementDAO.sumMilesAccruedBetween(
				memberNumber, from, when);
		AccountReport r = new AccountReport();
		r.setMiles(totalMiles);
		r.setEliteStatus(EliteStatus.fromMiles(pastYearMiles));
		r.setPastYearAccruedMiles(pastYearMiles);
		r.setNextMonthEliteStatus(EliteStatus.fromMiles(nextMiles));
		r.setPast11MonthsAccruedMiles(nextMiles);
		return r;
	}

	private EliteStatus getEliteStatus(long memberId, Date when) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(when);
		cal.set(Calendar.HOUR, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		when = cal.getTime();

		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.add(Calendar.YEAR, -1);
		Date from = cal.getTime();

		int miles = memberAccountStatementDAO.sumMilesAccruedBetween(memberId,
				from, when);

		return EliteStatus.fromMiles(miles);
	}

	@Override
	public void addSegment(Segment segment) {
		// We tolerate duplicates, as this is a file-based system.
		if (segmentDAO.exists(segment)) {
			// TODO log
			return;
		}
		Long statementId = null;
		try {
			long memberId = memberDAO.getIdByMemberNumber(segment
					.getLoadedMemberNumber());
			statementId = processSegment(memberId, segment);
		} catch (MemberNotFoundException mnfe) {
			// Member was not found - no statement will be created
			statementId = null;
		}
		// Member was found - create statement for his/her account
		long segmentId = segmentDAO.insert(segment, statementId);
	}

	protected long processSegment(long memberId, Segment segment) {
		Accrual accrual = accrualService.lookup(segment.getDeparture(),
				segment.getOrigin(), segment.getDestination(),
				segment.getAirline(), segment.getBookingClass());
		EliteStatus eliteStatus = getEliteStatus(memberId,
				segment.getDeparture());
		float factor = eliteBonusService.lookup(eliteStatus);
		int miles = (int) (factor * accrual.getMiles()) + accrual.getMiles();
		return createStatement(memberId, segment.getDeparture(), miles);
	}

	// @Scheduled(cron = "*/30 * * * *")
	protected void processOrphanedSegments() {
		for (Segment s : segmentDAO.findOrphaned()) {
			// TODO implement
		}
	}

	private long createStatement(long memberId, Date date, int miles) {
		return memberAccountStatementDAO.insert(memberId, date, miles);
	}

	@Override
	public FlightsReport getFlightsReport(long memberId, Date flightsFrom,
			Date flightsUntil) {
		MemberNumber memberNumber = memberDAO.getMemberNumberByID(memberId);
		return getFlightsReport(memberNumber, flightsFrom, flightsUntil);
	}

	@Override
	public FlightsReport getFlightsReport(MemberNumber memberNumber, Date from,
			Date until) {
		Calendar c = Calendar.getInstance();

		// By default show past 24 months
		if (until == null) {
			until = c.getTime();
		}

		if (from == null) {
			c.setTime(until);
			c.add(Calendar.MONTH, -24);
			from = c.getTime();
		}

		// Reset boundaries
		c.setTime(from);
		c.set(Calendar.HOUR, 0);
		c.set(Calendar.MINUTE, 0);
		c.set(Calendar.SECOND, 0);
		from = c.getTime();

		c.setTime(until);
		c.set(Calendar.HOUR, 23);
		c.set(Calendar.MINUTE, 59);
		c.set(Calendar.SECOND, 59);
		until = c.getTime();

		FlightsReport report = new FlightsReport();

		report.setSegments(segmentDAO.find(memberNumber, from, until));
		report.setFrom(from);
		report.setUntil(until);
		return report;

	}

	@Override
	public void modifySegment(long memberId, long segmentId,
			AirlineCode airline, int flightNumber, AirportCode origin,
			AirportCode destination, Date departure, int miles) {
		segmentDAO.update(memberId, segmentId, airline, flightNumber, origin,
				destination, departure);
		memberAccountStatementDAO.updateMilesBySegment(segmentId, miles);
	}

	@Override
	public void addNote(long memberId, String note) {
		noteDAO.addNote(memberId, note);
	}

	@Override
	public List<Note> getNotes(long memberId) {
		return noteDAO.findNotes(memberId);
	}

	@Override
	public Segment getSegment(long memberId, long segmentId) {
		return segmentDAO.get(memberId, segmentId);
	}
}
