package com.flozano.mim.member.impl;

import java.util.List;

import javax.inject.Inject;

import org.displaytag.pagination.PaginatedList;

import org.springframework.mail.MailSender;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.flozano.mim.MemberNumber;
import com.flozano.mim.member.EmailAddress;
import com.flozano.mim.member.FullName;
import com.flozano.mim.member.Member;
import com.flozano.mim.member.MemberNotFoundException;
import com.flozano.mim.member.MemberNumberGenerator;
import com.flozano.mim.member.MemberService;
import com.flozano.mim.member.Password;
import com.flozano.mim.member.PhoneNumber;
import com.flozano.mim.member.PostalAddress;
import com.flozano.mim.member.dao.MemberDAO;
import com.flozano.mim.util.Check;
import com.flozano.mim.util.PaginatedListBean;
import com.flozano.mim.util.SortDirection;

@Transactional
@Service("memberService")
public class MemberServiceImpl implements MemberService {

	private static final int PAGE_SIZE = 50;

	private final MemberDAO memberDAO;
	private final MailSender mailSender;
	private final MemberNumberGenerator memberNumberGenerator;

	@Inject
	public MemberServiceImpl(MemberDAO memberDAO,
			MemberNumberGenerator memberNumberGenerator, MailSender mailSender) {
		Check.notNull(memberDAO, memberNumberGenerator, mailSender);
		this.memberDAO = memberDAO;
		this.memberNumberGenerator = memberNumberGenerator;
		this.mailSender = mailSender;
	}

	@Override
	public MemberNumber register(FullName fullName, Password password,
			EmailAddress emailAddress, PhoneNumber phoneNumber,
			PostalAddress postalAddress) {
		Check.notNull(fullName);
		Check.notNull(password);
		Check.notNull(emailAddress);
		Check.notNull(phoneNumber);
		Check.notNull(postalAddress);

		MemberNumber memberNumber = memberNumberGenerator.generateNew();
		long id = memberDAO.insert(memberNumber, fullName, password,
				emailAddress, phoneNumber, postalAddress);

		sendMessage(fullName, emailAddress, memberNumber);
		return memberNumber;
	}

	private void sendMessage(FullName fullName, EmailAddress emailAddress,
			MemberNumber memberNumber) {
		SimpleMailMessage m = new SimpleMailMessage();
		m.setSubject("Welcome to Milles i Més!");
		m.setText("Dear " + fullName
				+ ", Welcome to Milles i Més. \nYour Member number is: "
				+ memberNumber);
		m.setTo(emailAddress.toString());
		mailSender.send(m);
	}

	@Override
	public void changePassword(MemberNumber memberNumber, Password oldPassword,
			Password newPassword) {
		int rows = memberDAO.updatePassword(memberNumber, oldPassword,
				newPassword);
		if (rows != 1) {
			throw new BadPasswordException(oldPassword);
		}
	}

	@Override
	public Member getMember(MemberNumber memberNumber)
			throws MemberNotFoundException {
		return memberDAO.get(memberNumber);
	}

	@Override
	public Member getMember(long memberId) throws MemberNotFoundException {
		return memberDAO.get(memberId);
	}

	@Override
	public void update(MemberNumber memberNumber, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress address)
			throws MemberNotFoundException {

		Member m = memberDAO.get(memberNumber);
		memberDAO.update(memberNumber, emailAddress, phoneNumber, address);

		if (!m.getEmailAddress().equals(emailAddress)) {
			sendMessage(m.getFullName(), emailAddress, memberNumber);
		}
	}

	@Override
	public void update(long memberId, FullName fullName, EmailAddress emailAddress,
			PhoneNumber phoneNumber, PostalAddress postalAddress) {
		memberDAO.update(memberId, fullName, emailAddress, phoneNumber,
				postalAddress);
	}

	@Override
	public PaginatedList find(String search, String sortProperty,
			SortDirection direction, int page) {
		int offset = (page - 1) * PAGE_SIZE;
		int count = memberDAO.count(search);
		List<Member> members = memberDAO.find(search, sortProperty, direction,
				PAGE_SIZE, offset);
		return new PaginatedListBean<Member>(members, page, PAGE_SIZE, count,
				sortProperty, direction);
	}

}
