package com.flozano.mim.web.controllers.admin;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.inject.Inject;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import javax.xml.bind.ValidationException;

import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.flozano.mim.AppException;
import com.flozano.mim.air.AirService;
import com.flozano.mim.air.AirlineCode;
import com.flozano.mim.air.AirportCode;
import com.flozano.mim.air.Segment;
import com.flozano.mim.member.AccountService;
import com.flozano.mim.member.EmailAddress;
import com.flozano.mim.member.FullName;
import com.flozano.mim.member.Member;
import com.flozano.mim.member.MemberService;
import com.flozano.mim.member.PhoneNumber;
import com.flozano.mim.member.PostalAddress;
import com.flozano.mim.util.Check;
import com.flozano.mim.util.SortDirection;

@Controller
@RequestMapping("/admin/members")
public class MemberController {

	private final Map<String, String> sortPropertiesToColumns = new HashMap<String, String>();

	private final MemberService memberService;

	private final AccountService accountService;
	private final AirService airService;

	@Inject
	public MemberController(MemberService memberService,
			AccountService accountService, AirService airService) {
		Check.notNull(memberService, accountService, airService);
		this.memberService = memberService;
		this.accountService = accountService;
		this.airService = airService;
		// TODO clean this
		sortPropertiesToColumns.put("memberNumber", "member_number");
		sortPropertiesToColumns.put("fullName.firstName", "first_name");
		sortPropertiesToColumns.put("fullName.lastName", "last_names");
		sortPropertiesToColumns.put("postalAddress.country", "country");
	}

	@RequestMapping(method = RequestMethod.GET)
	public String list(
			Model model,
			@RequestParam(required = false, defaultValue = "%") String search,
			@RequestParam(required = false, defaultValue = "memberNumber") String sort,
			@RequestParam(required = false, defaultValue = "asc") SortDirection dir,
			@RequestParam(required = false, defaultValue = "1") int page) {
		sort = sort != null && sortPropertiesToColumns.containsKey(sort) ? sortPropertiesToColumns
				.get(sort) : sort;
		model.addAttribute("members", memberService.find(search == null
				|| search.trim().equals("") ? "%" : search, sort, dir, page));
		return "admin/members/list";
	}

	public static class UpdateMemberForm {
		private FullName fullName = new FullName();
		private PostalAddress postalAddress = new PostalAddress();
		private String emailAddress;
		private String phoneNumber;

		public UpdateMemberForm() {
		}

		public UpdateMemberForm(Member member) {
			this.fullName = member.getFullName();
			this.emailAddress = member.getEmailAddress().toString();
			this.phoneNumber = member.getPhoneNumber().toString();
			this.postalAddress = member.getPostalAddress();
		}

		public FullName getFullName() {
			return fullName;
		}

		public void setFullName(FullName fullName) {
			this.fullName = fullName;
		}

		public PostalAddress getPostalAddress() {
			return postalAddress;
		}

		public void setPostalAddress(PostalAddress postalAddress) {
			this.postalAddress = postalAddress;
		}

		public String getEmailAddress() {
			return emailAddress;
		}

		public void setEmailAddress(String emailAddress) {
			this.emailAddress = emailAddress;
		}

		public String getPhoneNumber() {
			return phoneNumber;
		}

		public void setPhoneNumber(String phoneNumber) {
			this.phoneNumber = phoneNumber;
		}
	}

	@RequestMapping(method = RequestMethod.GET, value = "/{memberId}")
	public String getMember(
			Model model,
			@PathVariable("memberId") long memberId,
			@RequestParam(required = false, defaultValue = "false") Boolean memberUpdated,
			@RequestParam(required = false, defaultValue = "2000-01-01") @DateTimeFormat(pattern = "yyyy-MM-dd") Date from,
			@RequestParam(required = false, defaultValue = "2050-12-31") @DateTimeFormat(pattern = "yyyy-MM-dd") Date until) {
		model.addAttribute("memberForm",
				new UpdateMemberForm(memberService.getMember(memberId)));
		model.addAttribute("memberUpdated", memberUpdated);
		model.addAttribute("flightsReport",
				accountService.getFlightsReport(memberId, from, until));
		model.addAttribute("notes", accountService.getNotes(memberId));
		return "admin/members/edit";
	}

	@RequestMapping(method = RequestMethod.POST, value = "/{memberId}")
	public String updateMember(
			@ModelAttribute("memberForm") UpdateMemberForm form,
			BindingResult bindingResult,
			RedirectAttributes redirectAttributes,
			Model model,
			@PathVariable long memberId,
			@RequestParam(required = false, defaultValue = "2000-01-01") @DateTimeFormat(pattern = "yyyy-MM-dd") Date from,
			@RequestParam(required = false, defaultValue = "2050-12-31") @DateTimeFormat(pattern = "yyyy-MM-dd") Date until) {
		if (!bindingResult.hasErrors()) {
			try {
				memberService.update(memberId, form.getFullName(),
						new EmailAddress(form.getEmailAddress()),
						new PhoneNumber(form.getPhoneNumber()),
						form.getPostalAddress());
				redirectAttributes.addAttribute("memberUpdated", Boolean.TRUE);
				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
				redirectAttributes.addAttribute("from", sdf.format(from));
				redirectAttributes.addAttribute("until", sdf.format(until));
				return "redirect:/admin/members/{memberId}";
			} catch (AppException e) {
				bindingResult.addError(new ObjectError("memberForm", e
						.getLocalizedMessage()));
			}
		}
		model.addAttribute("flightsReport",
				accountService.getFlightsReport(memberId, from, until));
		model.addAttribute("notes", accountService.getNotes(memberId));
		return "admin/members/edit";
	}

	@RequestMapping(method = RequestMethod.POST, value = "/{memberId}/notes")
	public String addNote(Model model, @PathVariable long memberId,
			@RequestParam(required = false, defaultValue = "") String from,
			@RequestParam(required = false, defaultValue = "") String until,
			@RequestParam(required = true) String content,
			RedirectAttributes redirect) {
		// model.addAttribute("memberId", memberId);
		accountService.addNote(memberId, content);
		redirect.addAttribute("from", from);
		redirect.addAttribute("until", until);
		return "redirect:/admin/members/{memberId}";
	}

	@RequestMapping(method = RequestMethod.GET, value = "/{memberId}/flights/{flightId}")
	public String getFlight(Model model, @PathVariable long memberId,
			@PathVariable long flightId, RedirectAttributes redirect) {
		Segment segment = accountService.getSegment(memberId, flightId);
		FlightUpdateForm f = new FlightUpdateForm(segment);
		model.addAttribute("flightForm", f);
		model.addAttribute("airlines", airService.findAllAirlines());
		model.addAttribute("airports", airService.findAllAirports());
		model.addAttribute("memberId", memberId);
		return "admin/members/flight";
	}

	@RequestMapping(method = RequestMethod.POST, value = "/{memberId}/flights/{flightId}")
	public String updateFlight(@ModelAttribute FlightUpdateForm form,
			@PathVariable long memberId, @PathVariable long flightId,
			RedirectAttributes redirect, BindingResult result, Model model) {
		if (!result.hasErrors()) {
			try {
				accountService.modifySegment(memberId, flightId,
						new AirlineCode(form.getAirline()),
						form.getFlightNumber(),
						new AirportCode(form.getOrigin()),
						new AirportCode(form.getDestination()),
						form.getDeparture(), form.getMiles());
				return "redirect:/admin/members/{memberId}";
			} catch (AppException e) {
				result.addError(new ObjectError("memberForm", e
						.getLocalizedMessage()));
			}
		}
		model.addAttribute("airlines", airService.findAllAirlines());
		model.addAttribute("airports", airService.findAllAirports());
		model.addAttribute("memberId", memberId);

		return "admin/members/flight";
	}

	public static class FlightUpdateForm {

		private SimpleDateFormat sdf = new SimpleDateFormat(
				"yyyy-MM-dd");

		@NotNull
		private Date departure;
		@NotNull
		@Size(min = 3, max = 3)
		private String origin;
		@NotNull
		@Size(min = 3, max = 3)
		private String destination;
		@NotNull
		@Size(min = 2, max = 2)
		private String airline;
		private int flightNumber = 0;
		@NotNull
		@Size(min = 1, max = 1)
		private String bookingClass;
		private int miles = 0;

		public FlightUpdateForm() {
		}
		
		public String getDepartureText() {
			return sdf.format(departure);
		}
		
		public void setDepartureText(String text) throws ValidationException {
			try {
				this.departure = sdf.parse(text);
			} catch (ParseException e) {
				throw new ValidationException("Invalid date");
			}
		}
		
		public FlightUpdateForm(Segment segment) {
			departure = segment.getDeparture();
			origin = segment.getOrigin().toString();
			destination = segment.getDestination().toString();
			airline = segment.getAirline().toString();
			flightNumber = segment.getFlightNumber();
			bookingClass = segment.getBookingClass().toString();
			miles = segment.getMiles();
		}

		public int getMiles() {
			return miles;
		}

		public void setMiles(int miles) {
			this.miles = miles;
		}

		public Date getDeparture() {
			return departure;
		}

		public void setDeparture(Date departure) {
			this.departure = departure;
		}

		public String getOrigin() {
			return origin;
		}

		public void setOrigin(String origin) {
			this.origin = origin;
		}

		public String getDestination() {
			return destination;
		}

		public void setDestination(String destination) {
			this.destination = destination;
		}

		public String getAirline() {
			return airline;
		}

		public void setAirline(String airline) {
			this.airline = airline;
		}

		public int getFlightNumber() {
			return flightNumber;
		}

		public void setFlightNumber(int flightNumber) {
			this.flightNumber = flightNumber;
		}

		public String getBookingClass() {
			return bookingClass;
		}

		public void setBookingClass(String bookingClass) {
			this.bookingClass = bookingClass;
		}

	}

}
